package sun.font;

import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.Stroke;
import java.awt.BasicStroke;
import java.awt.font.TextAttribute;
import java.awt.Shape;
import java.awt.Graphics2D;
import java.util.concurrent.ConcurrentHashMap;

abstract class Underline
{
    private static final float DEFAULT_THICKNESS = 1.0f;
    private static final boolean USE_THICKNESS = true;
    private static final boolean IGNORE_THICKNESS = false;
    private static final ConcurrentHashMap<Object, Underline> UNDERLINES;
    private static final Underline[] UNDERLINE_LIST;
    
    Underline() {
    }
    
    abstract void drawUnderline(Graphics2D p0, float p1, float p2, float p3, float p4);
    
    abstract float getLowerDrawLimit(float p0);
    
    abstract Shape getUnderlineShape(float p0, float p1, float p2, float p3);
    
    static Underline getUnderline(Object key) {
        if (key == null) {
            return null;
        }
        return (Underline)Underline.UNDERLINES.get(key);
    }
    
    static Underline getUnderline(int n) {
        return (n < 0) ? null : Underline.UNDERLINE_LIST[n];
    }
    
    static {
        UNDERLINES = new ConcurrentHashMap(6);
        Underline[] underline_LIST = new Underline[6];
        underline_LIST[0] = new Underline.StandardUnderline(0.0f, 1.0f, null, true);
        Underline.UNDERLINES.put(TextAttribute.UNDERLINE_ON, underline_LIST[0]);
        underline_LIST[1] = new Underline.StandardUnderline(1.0f, 1.0f, null, false);
        Underline.UNDERLINES.put(TextAttribute.UNDERLINE_LOW_ONE_PIXEL, underline_LIST[1]);
        underline_LIST[2] = new Underline.StandardUnderline(1.0f, 2.0f, null, false);
        Underline.UNDERLINES.put(TextAttribute.UNDERLINE_LOW_TWO_PIXEL, underline_LIST[2]);
        underline_LIST[3] = new Underline.StandardUnderline(1.0f, 1.0f, new float[] { 1.0f, 1.0f }, false);
        Underline.UNDERLINES.put(TextAttribute.UNDERLINE_LOW_DOTTED, underline_LIST[3]);
        underline_LIST[4] = new Underline.IMGrayUnderline();
        Underline.UNDERLINES.put(TextAttribute.UNDERLINE_LOW_GRAY, underline_LIST[4]);
        underline_LIST[5] = new Underline.StandardUnderline(1.0f, 1.0f, new float[] { 4.0f, 4.0f }, false);
        Underline.UNDERLINES.put(TextAttribute.UNDERLINE_LOW_DASHED, underline_LIST[5]);
        UNDERLINE_LIST = underline_LIST;
    }
    
    private static final class StandardUnderline extends Underline
    {
        private float shift;
        private float thicknessMultiplier;
        private float[] dashPattern;
        private boolean useThickness;
        private BasicStroke cachedStroke;
        
        StandardUnderline(float shift, float thicknessMultiplier, float[] dashPattern, boolean useThickness) {
            this.shift = shift;
            this.thicknessMultiplier = thicknessMultiplier;
            this.dashPattern = dashPattern;
            this.useThickness = useThickness;
            this.cachedStroke = null;
        }
        
        private BasicStroke createStroke(float n) {
            if (this.dashPattern == null) {
                return new BasicStroke(n, 0, 0);
            }
            return new BasicStroke(n, 0, 0, 10.0f, this.dashPattern, 0.0f);
        }
        
        private float getLineThickness(float n) {
            if (this.useThickness) {
                return n * this.thicknessMultiplier;
            }
            return 1.0f * this.thicknessMultiplier;
        }
        
        private Stroke getStroke(float n) {
            float lineThickness = this.getLineThickness(n);
            BasicStroke cachedStroke = this.cachedStroke;
            if (cachedStroke == null || cachedStroke.getLineWidth() != lineThickness) {
                cachedStroke = this.createStroke(lineThickness);
                this.cachedStroke = cachedStroke;
                return cachedStroke;
            }
            return cachedStroke;
        }
        
        @Override
        void drawUnderline(Graphics2D graphics2D, float n, float x1, float x2, float n2) {
            Stroke stroke = graphics2D.getStroke();
            graphics2D.setStroke(this.getStroke(n));
            graphics2D.draw(new Line2D.Float(x1, n2 + this.shift, x2, n2 + this.shift));
            graphics2D.setStroke(stroke);
        }
        
        @Override
        float getLowerDrawLimit(float n) {
            return this.shift + this.getLineThickness(n);
        }
        
        @Override
        Shape getUnderlineShape(float n, float x1, float x2, float n2) {
            return this.getStroke(n).createStrokedShape(new Line2D.Float(x1, n2 + this.shift, x2, n2 + this.shift));
        }
    }
    
    private static class IMGrayUnderline extends Underline
    {
        private BasicStroke stroke;
        
        IMGrayUnderline() {
            this.stroke = new BasicStroke(1.0f, 0, 0, 10.0f, new float[] { 1.0f, 1.0f }, 0.0f);
        }
        
        @Override
        void drawUnderline(Graphics2D graphics2D, float n, float x1, float x2, float n2) {
            Stroke stroke = graphics2D.getStroke();
            graphics2D.setStroke(this.stroke);
            Line2D.Float float1 = new Line2D.Float(x1, n2, x2, n2);
            graphics2D.draw(float1);
            ++float1.y1;
            ++float1.y2;
            ++float1.x1;
            graphics2D.draw(float1);
            graphics2D.setStroke(stroke);
        }
        
        @Override
        float getLowerDrawLimit(float n) {
            return 2.0f;
        }
        
        @Override
        Shape getUnderlineShape(float n, float x1, float x2, float n2) {
            GeneralPath generalPath = new GeneralPath();
            Line2D.Float float1 = new Line2D.Float(x1, n2, x2, n2);
            generalPath.append(this.stroke.createStrokedShape(float1), false);
            ++float1.y1;
            ++float1.y2;
            ++float1.x1;
            generalPath.append(this.stroke.createStrokedShape(float1), false);
            return generalPath;
        }
    }
}
